package com.topsail.uartBackend.config;

import com.topsail.uartBackend.core.CodeMsg;
import com.topsail.uartBackend.core.Result;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.validation.BindException;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.ParseException;
import java.util.List;

/**
 * <描述>
 *
 * @author lj
 * @date 2022/07/21  10:09
 */
@ControllerAdvice
@ResponseBody
public class GlobalException {

    @ExceptionHandler(value = Exception.class)//拦截所有异常
    public Result<String> exceptionHandler(HttpServletRequest request, Exception e){
        e.printStackTrace();
        if(e instanceof BindException) {
            BindException ex = (BindException)e;
            List<ObjectError> errors = ex.getAllErrors();//绑定错误返回很多错误，是一个错误列表，只需要第一个错误
            ObjectError error = errors.get(0);
            String msg = error.getDefaultMessage();
            return Result.error(CodeMsg.BIND_ERROR.fillArgs(msg));//给状态码填充参数
        }else if(e instanceof ParseException){
            return Result.error(CodeMsg.TIME_ERROR);
        }else if(e instanceof DuplicateKeyException){
            return Result.error(CodeMsg.DUPLICATEKEY_ERROR);
        }else if(e instanceof  IllegalStateException){
            return Result.error(CodeMsg.TOKEN_INVALID);
        }else {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            return Result.error(new CodeMsg(500,sw.toString().substring(0,300)));//输出错误的堆栈信息
        }

    }
}
